@@ -10,6 +10,7 @@
 var EventEmitter = require('events').EventEmitter
 var util = require('util')
 var utils = require('./utils')
+var sasl = require('./sasl')
 var pgPass = require('pgpass')
 var TypeOverrides = require('./type-overrides')
 
@@ -43,6 +44,7 @@ var Client = function (config) {
     stream: c.stream,
     ssl: this.connectionParameters.ssl,
     keepAlive: c.keepAlive || false,
+    keepAliveInitialDelayMillis: c.keepAliveInitialDelayMillis || 0,
     encoding: this.connectionParameters.client_encoding || 'utf8'
   })
   this.queryQueue = []
@@ -50,6 +52,7 @@ var Client = function (config) {
   this.processID = null
   this.secretKey = null
   this.ssl = this.connectionParameters.ssl || false
+  this._connectionTimeoutMillis = c.connectionTimeoutMillis || 0
 }
 
 util.inherits(Client, EventEmitter)
@@ -82,6 +85,14 @@ Client.prototype._connect = function (callback) {
   }
   this._connecting = true
 
+  var connectionTimeoutHandle
+  if (this._connectionTimeoutMillis > 0) {
+    connectionTimeoutHandle = setTimeout(() => {
+      con._ending = true
+      con.stream.destroy(new Error('timeout expired'))
+    }, this._connectionTimeoutMillis)
+  }
+
   if (this.host && this.host.indexOf('/') === 0) {
     con.connect(this.host + '/.s.PGSQL.' + this.port)
   } else {
@@ -103,7 +114,24 @@ Client.prototype._connect = function (callback) {
 
   function checkPgPass (cb) {
     return function (msg) {
-      if (self.password !== null) {
+      if (typeof self.password === 'function') {
+        self._Promise.resolve()
+          .then(() => self.password())
+          .then(pass => {
+            if (pass !== undefined) {
+              if (typeof pass !== 'string') {
+                con.emit('error', new TypeError('Password must be a string'))
+                return
+              }
+              self.connectionParameters.password = self.password = pass
+            } else {
+              self.connectionParameters.password = self.password = null
+            }
+            cb(msg)
+          }).catch(err => {
+            con.emit('error', err)
+          })
+      } else if (self.password !== null) {
         cb(msg)
       } else {
         pgPass(self.connectionParameters, function (pass) {
@@ -126,6 +154,28 @@ Client.prototype._connect = function (callback) {
     con.password(utils.postgresMd5PasswordHash(self.user, self.password, msg.salt))
   }))
 
+  // password request handling (SASL)
+  var saslSession
+  con.on('authenticationSASL', checkPgPass(function (msg) {
+    saslSession = sasl.startSession(msg.mechanisms)
+
+    con.sendSASLInitialResponseMessage(saslSession.mechanism, saslSession.response)
+  }))
+
+  // password request handling (SASL)
+  con.on('authenticationSASLContinue', function (msg) {
+    sasl.continueSession(saslSession, self.password, msg.data)
+
+    con.sendSCRAMClientFinalMessage(saslSession.response)
+  })
+
+  // password request handling (SASL)
+  con.on('authenticationSASLFinal', function (msg) {
+    sasl.finalizeSession(saslSession, msg.data)
+
+    saslSession = null
+  })
+
   con.once('backendKeyData', function (msg) {
     self.processID = msg.processID
     self.secretKey = msg.secretKey
@@ -136,6 +186,7 @@ Client.prototype._connect = function (callback) {
       return
     }
     this._connectionError = true
+    clearTimeout(connectionTimeoutHandle)
     if (callback) {
       return callback(err)
     }
@@ -173,6 +224,7 @@ Client.prototype._connect = function (callback) {
     con.removeListener('errorMessage', connectingErrorHandler)
     con.on('error', connectedErrorHandler)
     con.on('errorMessage', connectedErrorMessageHandler)
+    clearTimeout(connectionTimeoutHandle)
 
     // process possible callback argument to Client#connect
     if (callback) {
@@ -257,11 +309,13 @@ Client.prototype._attachListeners = function (con) {
   })
 
   // delegate portalSuspended to active query
+  // eslint-disable-next-line no-unused-vars
   con.on('portalSuspended', function (msg) {
     self.activeQuery.handlePortalSuspended(con)
   })
 
-  // deletagate emptyQuery to active query
+  // delegate emptyQuery to active query
+  // eslint-disable-next-line no-unused-vars
   con.on('emptyQuery', function (msg) {
     self.activeQuery.handleEmptyQuery(con)
   })
@@ -274,12 +328,14 @@ Client.prototype._attachListeners = function (con) {
   // if a prepared statement has a name and properly parses
   // we track that its already been executed so we don't parse
   // it again on the same client
+  // eslint-disable-next-line no-unused-vars
   con.on('parseComplete', function (msg) {
     if (self.activeQuery.name) {
-      con.parsedStatements[self.activeQuery.name] = true
+      con.parsedStatements[self.activeQuery.name] = self.activeQuery.text
     }
   })
 
+  // eslint-disable-next-line no-unused-vars
   con.on('copyInResponse', function (msg) {
     self.activeQuery.handleCopyInResponse(self.connection)
   })
@@ -455,8 +511,9 @@ Client.prototype.query = function (config, values, callback) {
   if (this.binary && !query.binary) {
     query.binary = true
   }
-  if (query._result) {
-    query._result._getTypeParser = this._types.getTypeParser.bind(this._types)
+
+  if (query._result && !query._result._types) {
+    query._result._types = this._types
   }
 
   if (!this._queryable) {
