@@ -21,6 +21,7 @@ var Connection = function (config) {
   config = config || {}
   this.stream = config.stream || new net.Socket()
   this._keepAlive = config.keepAlive
+  this._keepAliveInitialDelayMillis = config.keepAliveInitialDelayMillis
   this.lastBuffer = false
   this.lastOffset = 0
   this.buffer = null
@@ -47,25 +48,24 @@ var Connection = function (config) {
 util.inherits(Connection, EventEmitter)
 
 Connection.prototype.connect = function (port, host) {
+  var self = this
+
   if (this.stream.readyState === 'closed') {
     this.stream.connect(port, host)
   } else if (this.stream.readyState === 'open') {
     this.emit('connect')
   }
 
-  var self = this
-
   this.stream.on('connect', function () {
     if (self._keepAlive) {
-      self.stream.setKeepAlive(true)
+      self.stream.setKeepAlive(true, self._keepAliveInitialDelayMillis)
     }
     self.emit('connect')
   })
 
   const reportStreamError = function (error) {
-    // don't raise ECONNRESET errors - they can & should be ignored
-    // during disconnect
-    if (self._ending && error.code === 'ECONNRESET') {
+    // errors about disconnections should be ignored during disconnect
+    if (self._ending && (error.code === 'ECONNRESET' || error.code === 'EPIPE')) {
       return
     }
     self.emit('error', error)
@@ -91,9 +91,8 @@ Connection.prototype.connect = function (port, host) {
         return self.emit('error', new Error('There was an error establishing an SSL connection'))
     }
     var tls = require('tls')
-    self.stream = tls.connect({
+    const options = {
       socket: self.stream,
-      servername: host,
       checkServerIdentity: self.ssl.checkServerIdentity || tls.checkServerIdentity,
       rejectUnauthorized: self.ssl.rejectUnauthorized,
       ca: self.ssl.ca,
@@ -101,8 +100,13 @@ Connection.prototype.connect = function (port, host) {
       key: self.ssl.key,
       passphrase: self.ssl.passphrase,
       cert: self.ssl.cert,
+      secureOptions: self.ssl.secureOptions,
       NPNProtocols: self.ssl.NPNProtocols
-    })
+    }
+    if (net.isIP(host) === 0) {
+      options.servername = host
+    }
+    self.stream = tls.connect(options)
     self.attachListeners(self.stream)
     self.stream.on('error', reportStreamError)
 
@@ -190,6 +194,24 @@ Connection.prototype.password = function (password) {
   this._send(0x70, this.writer.addCString(password))
 }
 
+Connection.prototype.sendSASLInitialResponseMessage = function (mechanism, initialResponse) {
+  // 0x70 = 'p'
+  this.writer
+    .addCString(mechanism)
+    .addInt32(Buffer.byteLength(initialResponse))
+    .addString(initialResponse)
+
+  this._send(0x70)
+}
+
+Connection.prototype.sendSCRAMClientFinalMessage = function (additionalData) {
+  // 0x70 = 'p'
+  this.writer
+    .addString(additionalData)
+
+  this._send(0x70)
+}
+
 Connection.prototype._send = function (code, more) {
   if (!this.stream.writable) {
     return false
@@ -217,9 +239,11 @@ Connection.prototype.parse = function (query, more) {
   // normalize missing query names to allow for null
   query.name = query.name || ''
   if (query.name.length > 63) {
+    /* eslint-disable no-console */
     console.error('Warning! Postgres only supports 63 characters for query names.')
-    console.error('You supplied', query.name, '(', query.name.length, ')')
+    console.error('You supplied %s (%s)', query.name, query.name.length)
     console.error('This can cause conflicts and silent errors executing queries')
+    /* eslint-enable no-console */
   }
   // normalize null type array
   query.types = query.types || []
@@ -420,25 +444,53 @@ Connection.prototype.parseMessage = function (buffer) {
 }
 
 Connection.prototype.parseR = function (buffer, length) {
-  var code = 0
+  var code = this.parseInt32(buffer)
+
   var msg = new Message('authenticationOk', length)
-  if (msg.length === 8) {
-    code = this.parseInt32(buffer)
-    if (code === 3) {
-      msg.name = 'authenticationCleartextPassword'
-    }
-    return msg
-  }
-  if (msg.length === 12) {
-    code = this.parseInt32(buffer)
-    if (code === 5) { // md5 required
-      msg.name = 'authenticationMD5Password'
-      msg.salt = Buffer.alloc(4)
-      buffer.copy(msg.salt, 0, this.offset, this.offset + 4)
-      this.offset += 4
+
+  switch (code) {
+    case 0: // AuthenticationOk
+      return msg
+    case 3: // AuthenticationCleartextPassword
+      if (msg.length === 8) {
+        msg.name = 'authenticationCleartextPassword'
+        return msg
+      }
+      break
+    case 5: // AuthenticationMD5Password
+      if (msg.length === 12) {
+        msg.name = 'authenticationMD5Password'
+        msg.salt = Buffer.alloc(4)
+        buffer.copy(msg.salt, 0, this.offset, this.offset + 4)
+        this.offset += 4
+        return msg
+      }
+
+      break
+    case 10: // AuthenticationSASL
+      msg.name = 'authenticationSASL'
+      msg.mechanisms = []
+      do {
+        var mechanism = this.parseCString(buffer)
+
+        if (mechanism) {
+          msg.mechanisms.push(mechanism)
+        }
+      } while (mechanism)
+
+      return msg
+    case 11: // AuthenticationSASLContinue
+      msg.name = 'authenticationSASLContinue'
+      msg.data = this.readString(buffer, length - 4)
+
+      return msg
+    case 12: // AuthenticationSASLFinal
+      msg.name = 'authenticationSASLFinal'
+      msg.data = this.readString(buffer, length - 4)
+
       return msg
-    }
   }
+
   throw new Error('Unknown authenticationOk message type' + util.inspect(msg))
 }
 
@@ -554,7 +606,7 @@ Connection.prototype.parseE = function (buffer, length) {
     msg = new Error(fields.M)
     for (item in input) {
       // copy input properties to the error
-      if (input.hasOwnProperty(item)) {
+      if (Object.prototype.hasOwnProperty.call(input, item)) {
         msg[item] = input[item]
       }
     }
