@@ -148,6 +148,10 @@ Query.prototype.submit = function (connection) {
   if (typeof this.text !== 'string' && typeof this.name !== 'string') {
     return new Error('A query must have either text or a name. Supplying neither is unsupported.')
   }
+  const previous = connection.parsedStatements[this.name]
+  if (this.text && previous && this.text !== previous) {
+    return new Error(`Prepared statements must be unique - '${this.name}' was used for a different statement`)
+  }
   if (this.values && !Array.isArray(this.values)) {
     return new Error('Query values must be an array')
   }
@@ -190,7 +194,12 @@ Query.prototype.prepare = function (connection) {
   }
 
   if (self.values) {
-    self.values = self.values.map(utils.prepareValue)
+    try {
+      self.values = self.values.map(utils.prepareValue)
+    } catch (err) {
+      this.handleError(err, connection)
+      return
+    }
   }
 
   // http://developer.postgresql.org/pgdocs/postgres/protocol-flow.html#PROTOCOL-FLOW-EXT-QUERY
@@ -213,6 +222,7 @@ Query.prototype.handleCopyInResponse = function (connection) {
   connection.sendCopyFail('No source stream defined')
 }
 
+// eslint-disable-next-line no-unused-vars
 Query.prototype.handleCopyData = function (msg, connection) {
   // noop
 }
